# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .transaction_id import TransactionId
from .transaction_status import TransactionStatus
import datetime as dt
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class TransactionResponseBase(UniversalBaseModel):
    id: TransactionId
    status: TransactionStatus
    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .invoice_response_base import InvoiceResponseBase
from .invoice_id import InvoiceId
import typing
import datetime as dt
import pydantic
from .invoice_failure_type import InvoiceFailureType
from ...transaction.types.transaction_response_without_invoices import TransactionResponseWithoutInvoices
from ...vendor_credit_types.types.vendor_credit_id import VendorCreditId
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceResponse(InvoiceResponseBase):
    """
    Examples
    --------
    import datetime

    from mercoa.commons import Address, PhoneNumber
    from mercoa.entity_types import (
        ApprovalPolicyResponse,
        BusinessProfileResponse,
        CounterpartyCustomizationAccount,
        CounterpartyResponse,
        Ein,
        EntityUserResponse,
        IdentifierList_RolesList,
        ProfileResponse,
        Rule_Approver,
        TaxId,
        Trigger_Amount,
    )
    from mercoa.invoice_types import (
        ApprovalSlot,
        AssociatedApprovalAction,
        CommentResponse,
        InvoiceLineItemResponse,
        InvoiceResponse,
        PaymentDestinationOptions_Check,
    )
    from mercoa.payment_method_types import (
        PaymentMethodResponse_BankAccount,
        PaymentMethodResponse_Check,
    )
    from mercoa.transaction import (
        TransactionResponseWithoutInvoices_BankAccountToBankAccount,
    )

    InvoiceResponse(
        id="in_26e7b5d3-a739-4b23-9ad9-6aaa085f47a9",
        status="PAID",
        amount=100.0,
        currency="USD",
        invoice_date=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        deduction_date=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        settlement_date=datetime.datetime.fromisoformat(
            "2021-01-03 00:00:00+00:00",
        ),
        due_date=datetime.datetime.fromisoformat(
            "2021-01-31 00:00:00+00:00",
        ),
        invoice_number="INV-123",
        note_to_self="For the month of January",
        service_start_date=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        service_end_date=datetime.datetime.fromisoformat(
            "2021-01-31 00:00:00+00:00",
        ),
        payer_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
        payer=CounterpartyResponse(
            id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            foreign_id="MY-DB-ID-12345",
            name="Acme Inc.",
            email="customer@acme.com",
            accepted_tos=True,
            status="verified",
            is_customer=True,
            is_payor=True,
            is_payee=False,
            is_network_payor=False,
            is_network_payee=False,
            account_type="business",
            updated_at=datetime.datetime.fromisoformat(
                "2024-01-02 00:00:00+00:00",
            ),
            created_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
            profile=ProfileResponse(
                business=BusinessProfileResponse(
                    email="customer@acme.com",
                    legal_business_name="Acme Inc.",
                    business_type="llc",
                    phone=PhoneNumber(
                        country_code="1",
                        number="4155551234",
                    ),
                    address=Address(
                        address_line_1="123 Main St",
                        address_line_2="Unit 1",
                        city="San Francisco",
                        state_or_province="CA",
                        postal_code="94105",
                        country="US",
                    ),
                    tax_id_provided=True,
                    tax_id=TaxId(
                        ein=Ein(
                            number="12-3456789",
                        ),
                    ),
                    owners_provided=True,
                ),
            ),
            accounts=[
                CounterpartyCustomizationAccount(
                    account_id="85866843",
                    postal_code="94105",
                    name_on_account="John Doe",
                )
            ],
            payment_methods=[
                PaymentMethodResponse_BankAccount(
                    id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                    account_name="My Checking Account",
                    bank_name="Chase",
                    routing_number="12345678",
                    account_number="99988767623",
                    account_type="CHECKING",
                    status="VERIFIED",
                    is_default_source=True,
                    is_default_destination=True,
                    supported_currencies=["USD"],
                    metadata={},
                    frozen=False,
                    created_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                    updated_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                )
            ],
            counterparty_type=["ENTITY"],
        ),
        payment_source=PaymentMethodResponse_BankAccount(
            id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
            account_name="My Checking Account",
            bank_name="Chase",
            routing_number="12345678",
            account_number="99988767623",
            account_type="CHECKING",
            status="VERIFIED",
            is_default_source=True,
            is_default_destination=True,
            supported_currencies=["USD"],
            metadata={},
            frozen=False,
            created_at=datetime.datetime.fromisoformat(
                "2021-01-01 00:00:00+00:00",
            ),
            updated_at=datetime.datetime.fromisoformat(
                "2021-01-01 00:00:00+00:00",
            ),
        ),
        payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
        vendor_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
        vendor=CounterpartyResponse(
            id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
            foreign_id="MY-DB-ID-90909",
            name="Big Box Store",
            email="vendor@bigboxstore.com",
            accepted_tos=False,
            status="unverified",
            is_customer=False,
            is_payor=False,
            is_payee=True,
            is_network_payor=False,
            is_network_payee=False,
            account_type="business",
            updated_at=datetime.datetime.fromisoformat(
                "2024-01-02 00:00:00+00:00",
            ),
            created_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
            profile=ProfileResponse(
                business=BusinessProfileResponse(
                    email="vendor@bigboxstore.com",
                    legal_business_name="Big Box Store",
                    business_type="publicCorporation",
                    tax_id_provided=False,
                    owners_provided=False,
                ),
            ),
            payment_methods=[
                PaymentMethodResponse_BankAccount(
                    id="pm_7610541f-4619-4033-8620-cfccfb811293",
                    account_name="Vendor Checking Account",
                    bank_name="Chase",
                    routing_number="66554433",
                    account_number="55934059697648",
                    account_type="CHECKING",
                    status="NEW",
                    is_default_source=True,
                    is_default_destination=True,
                    supported_currencies=["USD"],
                    metadata={},
                    frozen=False,
                    created_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                    updated_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                )
            ],
            counterparty_type=["ENTITY"],
        ),
        payment_destination=PaymentMethodResponse_Check(
            id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
            pay_to_the_order_of="John Doe",
            address_line_1="123 Main St",
            address_line_2="Apt 1",
            city="New York",
            state_or_province="NY",
            postal_code="10001",
            country="US",
            is_default_source=False,
            is_default_destination=True,
            supported_currencies=["USD"],
            metadata={},
            frozen=False,
            created_at=datetime.datetime.fromisoformat(
                "2021-01-01 00:00:00+00:00",
            ),
            updated_at=datetime.datetime.fromisoformat(
                "2021-01-01 00:00:00+00:00",
            ),
        ),
        payment_destination_id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
        payment_destination_options=PaymentDestinationOptions_Check(
            delivery="MAIL",
            print_description=True,
        ),
        payment_destination_confirmed=True,
        transactions=[
            TransactionResponseWithoutInvoices_BankAccountToBankAccount(
                id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
                status="COMPLETED",
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
            )
        ],
        has_documents=True,
        has_source_email=True,
        comments=[
            CommentResponse(
                id="ic_b3525b66-da94-4525-9f31-426bcf657128",
                text="This is an approval comment",
                user=EntityUserResponse(
                    id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
                    foreign_id="MY-DB-ID-12345",
                    email="john.doe@acme.com",
                    name="John Doe",
                    roles=["admin", "approver"],
                    created_at=datetime.datetime.fromisoformat(
                        "2024-01-01 00:00:00+00:00",
                    ),
                    updated_at=datetime.datetime.fromisoformat(
                        "2024-01-01 00:00:00+00:00",
                    ),
                ),
                associated_approval_action=AssociatedApprovalAction(
                    user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                    action="APPROVE",
                ),
                created_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
            )
        ],
        line_items=[
            InvoiceLineItemResponse(
                id="inli_26672f38-eb9a-48f1-a7a0-f1b855e38cd7",
                amount=100.0,
                currency="USD",
                description="Product A",
                name="Product A",
                quantity=1.0,
                unit_price=100.0,
                category="EXPENSE",
                service_start_date=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
                service_end_date=datetime.datetime.fromisoformat(
                    "2021-01-31 00:00:00+00:00",
                ),
                metadata={"key1": "value1", "key2": "value2"},
                gl_account_id="600394",
                created_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
            )
        ],
        approvers=[
            ApprovalSlot(
                approval_policy_id="apvl_5ce50275-1789-42ea-bc60-bb7e6d03635c",
                approval_slot_id="inap_9bb311c9-7c15-4c9e-8148-63814e0abec6",
                assigned_user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                action="APPROVE",
                eligible_user_ids=["user_e24fc81c-c5ee-47e8-af42-4fe29d895506"],
                eligible_roles=["admin"],
                date=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
            )
        ],
        approval_policy=[
            ApprovalPolicyResponse(
                id="apvl_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                trigger=[
                    Trigger_Amount(
                        amount=100.0,
                        currency="USD",
                    )
                ],
                rule=Rule_Approver(
                    num_approvers=2,
                    identifier_list=IdentifierList_RolesList(
                        value=["Admin", "Controller"]
                    ),
                ),
                upstream_policy_id="root",
                updated_at=datetime.datetime.fromisoformat(
                    "2024-01-02 00:00:00+00:00",
                ),
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
            )
        ],
        metadata={"key1": "value1", "key2": "value2"},
        foreign_id="YOUR-DATABASE-ID",
        creator_user=EntityUserResponse(
            id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
            foreign_id="MY-DB-ID-12345",
            email="john.doe@acme.com",
            name="John Doe",
            roles=["admin", "approver"],
            created_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
            updated_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
        ),
        processed_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        created_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
    )
    """

    id: InvoiceId
    processed_at: typing.Optional[dt.datetime] = pydantic.Field(alias="processedAt", default=None)
    """
    Date when the invoice payment was processed.
    """

    settlement_date: typing.Optional[dt.datetime] = pydantic.Field(alias="settlementDate", default=None)
    """
    Date of funds settlement.
    """

    foreign_id: typing.Optional[str] = pydantic.Field(alias="foreignId", default=None)
    """
    The ID used to identify this invoice in your system. This ID must be unique within each creatorEntity in your system, e.g. two invoices with the same creatorEntity may not have the same foreign ID.
    """

    failure_type: typing.Optional[InvoiceFailureType] = pydantic.Field(alias="failureType", default=None)
    """
    If the invoice failed to be paid, this field will be populated with the type of failure.
    """

    transactions: typing.Optional[typing.List[TransactionResponseWithoutInvoices]] = pydantic.Field(default=None)
    """
    Transactions associated with this invoice.
    """

    vendor_credit_ids: typing.Optional[typing.List[VendorCreditId]] = pydantic.Field(
        alias="vendorCreditIds", default=None
    )
    """
    The IDs of the vendor credits that are currently applied to this invoice.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceDateFilter = typing.Union[
    typing.Literal["INVOICE_DATE", "DEDUCTION_DATE", "DUE_DATE", "SETTLEMENT_DATE", "CREATED_AT", "UPDATED_AT"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .metadata_customization_request import MetadataCustomizationRequest
from .payment_method_customization_request import PaymentMethodCustomizationRequest
import pydantic
from .ocr_customization_request import OcrCustomizationRequest
from .notification_customization_request import NotificationCustomizationRequest
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class EntityCustomizationRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import (
        EntityCustomizationRequest,
        MetadataCustomizationRequest,
        NotificationCustomizationRequest,
        OcrCustomizationRequest,
        PaymentMethodCustomizationRequest,
    )

    EntityCustomizationRequest(
        metadata=[
            MetadataCustomizationRequest(
                key="my_custom_field",
                disabled=True,
            ),
            MetadataCustomizationRequest(
                key="my_other_field",
                disabled=False,
            ),
        ],
        payment_source=[
            PaymentMethodCustomizationRequest(
                type="bankAccount",
                disabled=True,
            ),
            PaymentMethodCustomizationRequest(
                type="custom",
                schema_id="cpms_7df2974a-4069-454c-912f-7e58ebe030fb",
                disabled=True,
            ),
        ],
        backup_disbursement=[
            PaymentMethodCustomizationRequest(
                type="check",
                disabled=True,
            )
        ],
        payment_destination=[
            PaymentMethodCustomizationRequest(
                type="bankAccount",
                disabled=True,
            ),
            PaymentMethodCustomizationRequest(
                type="check",
                disabled=True,
            ),
        ],
        ocr=OcrCustomizationRequest(
            line_items=True,
            invoice_metadata=True,
            line_item_metadata=True,
            line_item_gl_account_id=True,
            predict_metadata=True,
            tax_and_shipping_as_line_items=True,
        ),
        notifications=NotificationCustomizationRequest(
            assume_role="admin",
        ),
    )
    """

    metadata: typing.Optional[typing.List[MetadataCustomizationRequest]] = None
    payment_source: typing.Optional[typing.List[PaymentMethodCustomizationRequest]] = pydantic.Field(
        alias="paymentSource", default=None
    )
    backup_disbursement: typing.Optional[typing.List[PaymentMethodCustomizationRequest]] = pydantic.Field(
        alias="backupDisbursement", default=None
    )
    payment_destination: typing.Optional[typing.List[PaymentMethodCustomizationRequest]] = pydantic.Field(
        alias="paymentDestination", default=None
    )
    ocr: typing.Optional[OcrCustomizationRequest] = None
    notifications: typing.Optional[NotificationCustomizationRequest] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .onboarding_option import OnboardingOption
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CommonOnboardingOptions(UniversalBaseModel):
    terms_of_service: OnboardingOption = pydantic.Field(alias="termsOfService")
    email: OnboardingOption
    name: OnboardingOption
    address: OnboardingOption
    phone: OnboardingOption
    ten_ninety_nine: OnboardingOption = pydantic.Field(alias="tenNinetyNine")
    w_9: OnboardingOption = pydantic.Field(alias="w9")
    bank_statement: OnboardingOption = pydantic.Field(alias="bankStatement")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CalculateVendorCreditUsageResponse,
    FindVendorCreditResponse,
    VendorCreditId,
    VendorCreditRequest,
    VendorCreditResponse,
)

__all__ = [
    "CalculateVendorCreditUsageResponse",
    "FindVendorCreditResponse",
    "VendorCreditId",
    "VendorCreditRequest",
    "VendorCreditResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

DocumentType = typing.Union[
    typing.Literal["INVOICE", "TEN_NINETY_NINE", "W9", "CHECK", "BANK_STATEMENT", "OTHER"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from .calculate_vendor_credit_usage_response import CalculateVendorCreditUsageResponse
from .find_vendor_credit_response import FindVendorCreditResponse
from .vendor_credit_id import VendorCreditId
from .vendor_credit_request import VendorCreditRequest
from .vendor_credit_response import VendorCreditResponse

__all__ = [
    "CalculateVendorCreditUsageResponse",
    "FindVendorCreditResponse",
    "VendorCreditId",
    "VendorCreditRequest",
    "VendorCreditResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .invoice_event import InvoiceEvent
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class InvoiceEventsResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.invoice_types import (
        InvoiceEvent,
        InvoiceEventsResponse,
        InvoiceLineItemUpdateRequest,
        InvoiceUpdateRequest,
        PaymentDestinationOptions_Check,
    )

    InvoiceEventsResponse(
        data=[
            InvoiceEvent(
                webhook_ids=["webhook_12345"],
                user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                data=InvoiceUpdateRequest(
                    status="NEW",
                    amount=100.0,
                    currency="USD",
                    invoice_date=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                    due_date=datetime.datetime.fromisoformat(
                        "2021-01-31 00:00:00+00:00",
                    ),
                    invoice_number="INV-123",
                    note_to_self="For the month of January",
                    payer_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                    payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                    vendor_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                    payment_destination_id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
                    payment_destination_options=PaymentDestinationOptions_Check(
                        delivery="MAIL",
                        print_description=True,
                    ),
                    line_items=[
                        InvoiceLineItemUpdateRequest(
                            id="inli_26672f38-eb9a-48f1-a7a0-f1b855e38cd7",
                            amount=100.0,
                            currency="USD",
                            description="Product A",
                            name="Product A",
                            quantity=1.0,
                            unit_price=100.0,
                            category="EXPENSE",
                            service_start_date=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                            service_end_date=datetime.datetime.fromisoformat(
                                "2021-01-31 00:00:00+00:00",
                            ),
                            metadata={"key1": "value1", "key2": "value2"},
                            gl_account_id="600394",
                        )
                    ],
                    creator_entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                    creator_user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                ),
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
            )
        ],
        count=1,
    )
    """

    data: typing.List[InvoiceEvent]
    count: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    FindTransactionsResponse,
    TransactionFailureReason,
    TransactionId,
    TransactionResponse,
    TransactionResponseBankToBankBase,
    TransactionResponseBankToBankWithInvoices,
    TransactionResponseBankToMailedCheckBase,
    TransactionResponseBankToMailedCheckWithInvoices,
    TransactionResponseBase,
    TransactionResponseCustomWithInvoices,
    TransactionResponseWithoutInvoices,
    TransactionResponseWithoutInvoices_BankAccountToBankAccount,
    TransactionResponseWithoutInvoices_BankAccountToMailedCheck,
    TransactionResponseWithoutInvoices_Custom,
    TransactionResponseWithoutInvoices_OffPlatform,
    TransactionResponse_BankAccountToBankAccount,
    TransactionResponse_BankAccountToMailedCheck,
    TransactionResponse_Custom,
    TransactionResponse_OffPlatform,
    TransactionStatus,
    TransactionType,
)

__all__ = [
    "FindTransactionsResponse",
    "TransactionFailureReason",
    "TransactionId",
    "TransactionResponse",
    "TransactionResponseBankToBankBase",
    "TransactionResponseBankToBankWithInvoices",
    "TransactionResponseBankToMailedCheckBase",
    "TransactionResponseBankToMailedCheckWithInvoices",
    "TransactionResponseBase",
    "TransactionResponseCustomWithInvoices",
    "TransactionResponseWithoutInvoices",
    "TransactionResponseWithoutInvoices_BankAccountToBankAccount",
    "TransactionResponseWithoutInvoices_BankAccountToMailedCheck",
    "TransactionResponseWithoutInvoices_Custom",
    "TransactionResponseWithoutInvoices_OffPlatform",
    "TransactionResponse_BankAccountToBankAccount",
    "TransactionResponse_BankAccountToMailedCheck",
    "TransactionResponse_Custom",
    "TransactionResponse_OffPlatform",
    "TransactionStatus",
    "TransactionType",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .transaction_response import TransactionResponse
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class FindTransactionsResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.commons import Address, PhoneNumber
    from mercoa.entity_types import (
        ApprovalPolicyResponse,
        BusinessProfileResponse,
        CounterpartyCustomizationAccount,
        CounterpartyResponse,
        Ein,
        EntityUserResponse,
        IdentifierList_RolesList,
        ProfileResponse,
        Rule_Approver,
        TaxId,
        Trigger_Amount,
    )
    from mercoa.invoice_types import (
        ApprovalSlot,
        AssociatedApprovalAction,
        CommentResponse,
        InvoiceLineItemResponse,
        InvoiceResponse,
        PaymentDestinationOptions_Check,
    )
    from mercoa.payment_method_types import (
        PaymentMethodResponse_BankAccount,
        PaymentMethodResponse_Check,
    )
    from mercoa.transaction import (
        FindTransactionsResponse,
        TransactionFailureReason,
        TransactionResponse_BankAccountToBankAccount,
        TransactionResponse_BankAccountToMailedCheck,
        TransactionResponseWithoutInvoices_BankAccountToBankAccount,
    )

    FindTransactionsResponse(
        count=3,
        has_more=False,
        data=[
            TransactionResponse_BankAccountToBankAccount(
                id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
                status="COMPLETED",
                invoices=[
                    InvoiceResponse(
                        id="in_26e7b5d3-a739-4b23-9ad9-6aaa085f47a9",
                        status="PAID",
                        amount=100.0,
                        currency="USD",
                        invoice_date=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        deduction_date=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        settlement_date=datetime.datetime.fromisoformat(
                            "2021-01-03 00:00:00+00:00",
                        ),
                        due_date=datetime.datetime.fromisoformat(
                            "2021-01-31 00:00:00+00:00",
                        ),
                        invoice_number="INV-123",
                        note_to_self="For the month of January",
                        service_start_date=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        service_end_date=datetime.datetime.fromisoformat(
                            "2021-01-31 00:00:00+00:00",
                        ),
                        payer_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                        payer=CounterpartyResponse(
                            id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                            foreign_id="MY-DB-ID-12345",
                            name="Acme Inc.",
                            email="customer@acme.com",
                            accepted_tos=True,
                            status="verified",
                            is_customer=True,
                            is_payor=True,
                            is_payee=False,
                            is_network_payor=False,
                            is_network_payee=False,
                            account_type="business",
                            updated_at=datetime.datetime.fromisoformat(
                                "2024-01-02 00:00:00+00:00",
                            ),
                            created_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                            profile=ProfileResponse(
                                business=BusinessProfileResponse(
                                    email="customer@acme.com",
                                    legal_business_name="Acme Inc.",
                                    business_type="llc",
                                    phone=PhoneNumber(
                                        country_code="1",
                                        number="4155551234",
                                    ),
                                    address=Address(
                                        address_line_1="123 Main St",
                                        address_line_2="Unit 1",
                                        city="San Francisco",
                                        state_or_province="CA",
                                        postal_code="94105",
                                        country="US",
                                    ),
                                    tax_id_provided=True,
                                    tax_id=TaxId(
                                        ein=Ein(
                                            number="12-3456789",
                                        ),
                                    ),
                                    owners_provided=True,
                                ),
                            ),
                            accounts=[
                                CounterpartyCustomizationAccount(
                                    account_id="85866843",
                                    postal_code="94105",
                                    name_on_account="John Doe",
                                )
                            ],
                            payment_methods=[
                                PaymentMethodResponse_BankAccount(
                                    id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                                    account_name="My Checking Account",
                                    bank_name="Chase",
                                    routing_number="12345678",
                                    account_number="99988767623",
                                    account_type="CHECKING",
                                    status="VERIFIED",
                                    is_default_source=True,
                                    is_default_destination=True,
                                    supported_currencies=["USD"],
                                    metadata={},
                                    frozen=False,
                                    created_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                    updated_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                )
                            ],
                            counterparty_type=["ENTITY"],
                        ),
                        payment_source=PaymentMethodResponse_BankAccount(
                            id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                            account_name="My Checking Account",
                            bank_name="Chase",
                            routing_number="12345678",
                            account_number="99988767623",
                            account_type="CHECKING",
                            status="VERIFIED",
                            is_default_source=True,
                            is_default_destination=True,
                            supported_currencies=["USD"],
                            metadata={},
                            frozen=False,
                            created_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                            updated_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                        ),
                        payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                        vendor_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                        vendor=CounterpartyResponse(
                            id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                            foreign_id="MY-DB-ID-90909",
                            name="Big Box Store",
                            email="vendor@bigboxstore.com",
                            accepted_tos=False,
                            status="unverified",
                            is_customer=False,
                            is_payor=False,
                            is_payee=True,
                            is_network_payor=False,
                            is_network_payee=False,
                            account_type="business",
                            updated_at=datetime.datetime.fromisoformat(
                                "2024-01-02 00:00:00+00:00",
                            ),
                            created_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                            profile=ProfileResponse(
                                business=BusinessProfileResponse(
                                    email="vendor@bigboxstore.com",
                                    legal_business_name="Big Box Store",
                                    business_type="publicCorporation",
                                    tax_id_provided=False,
                                    owners_provided=False,
                                ),
                            ),
                            payment_methods=[
                                PaymentMethodResponse_BankAccount(
                                    id="pm_7610541f-4619-4033-8620-cfccfb811293",
                                    account_name="Vendor Checking Account",
                                    bank_name="Chase",
                                    routing_number="66554433",
                                    account_number="55934059697648",
                                    account_type="CHECKING",
                                    status="NEW",
                                    is_default_source=True,
                                    is_default_destination=True,
                                    supported_currencies=["USD"],
                                    metadata={},
                                    frozen=False,
                                    created_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                    updated_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                )
                            ],
                            counterparty_type=["ENTITY"],
                        ),
                        payment_destination=PaymentMethodResponse_Check(
                            id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
                            pay_to_the_order_of="John Doe",
                            address_line_1="123 Main St",
                            address_line_2="Apt 1",
                            city="New York",
                            state_or_province="NY",
                            postal_code="10001",
                            country="US",
                            is_default_source=False,
                            is_default_destination=True,
                            supported_currencies=["USD"],
                            metadata={},
                            frozen=False,
                            created_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                            updated_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                        ),
                        payment_destination_id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
                        payment_destination_options=PaymentDestinationOptions_Check(
                            delivery="MAIL",
                            print_description=True,
                        ),
                        payment_destination_confirmed=True,
                        transactions=[
                            TransactionResponseWithoutInvoices_BankAccountToBankAccount(
                                id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
                                status="COMPLETED",
                                created_at=datetime.datetime.fromisoformat(
                                    "2024-01-01 00:00:00+00:00",
                                ),
                                updated_at=datetime.datetime.fromisoformat(
                                    "2024-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        has_documents=True,
                        has_source_email=True,
                        comments=[
                            CommentResponse(
                                id="ic_b3525b66-da94-4525-9f31-426bcf657128",
                                text="This is an approval comment",
                                user=EntityUserResponse(
                                    id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
                                    foreign_id="MY-DB-ID-12345",
                                    email="john.doe@acme.com",
                                    name="John Doe",
                                    roles=["admin", "approver"],
                                    created_at=datetime.datetime.fromisoformat(
                                        "2024-01-01 00:00:00+00:00",
                                    ),
                                    updated_at=datetime.datetime.fromisoformat(
                                        "2024-01-01 00:00:00+00:00",
                                    ),
                                ),
                                associated_approval_action=AssociatedApprovalAction(
                                    user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                                    action="APPROVE",
                                ),
                                created_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                                updated_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        line_items=[
                            InvoiceLineItemResponse(
                                id="inli_26672f38-eb9a-48f1-a7a0-f1b855e38cd7",
                                amount=100.0,
                                currency="USD",
                                description="Product A",
                                name="Product A",
                                quantity=1.0,
                                unit_price=100.0,
                                category="EXPENSE",
                                service_start_date=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                                service_end_date=datetime.datetime.fromisoformat(
                                    "2021-01-31 00:00:00+00:00",
                                ),
                                metadata={"key1": "value1", "key2": "value2"},
                                gl_account_id="600394",
                                created_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                                updated_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        approvers=[
                            ApprovalSlot(
                                approval_policy_id="apvl_5ce50275-1789-42ea-bc60-bb7e6d03635c",
                                approval_slot_id="inap_9bb311c9-7c15-4c9e-8148-63814e0abec6",
                                assigned_user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                                action="APPROVE",
                                eligible_user_ids=[
                                    "user_e24fc81c-c5ee-47e8-af42-4fe29d895506"
                                ],
                                eligible_roles=["admin"],
                                date=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        approval_policy=[
                            ApprovalPolicyResponse(
                                id="apvl_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                                trigger=[
                                    Trigger_Amount(
                                        amount=100.0,
                                        currency="USD",
                                    )
                                ],
                                rule=Rule_Approver(
                                    num_approvers=2,
                                    identifier_list=IdentifierList_RolesList(
                                        value=["Admin", "Controller"]
                                    ),
                                ),
                                upstream_policy_id="root",
                                updated_at=datetime.datetime.fromisoformat(
                                    "2024-01-02 00:00:00+00:00",
                                ),
                                created_at=datetime.datetime.fromisoformat(
                                    "2024-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        metadata={"key1": "value1", "key2": "value2"},
                        foreign_id="YOUR-DATABASE-ID",
                        creator_user=EntityUserResponse(
                            id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
                            foreign_id="MY-DB-ID-12345",
                            email="john.doe@acme.com",
                            name="John Doe",
                            roles=["admin", "approver"],
                            created_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                            updated_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                        ),
                        processed_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        created_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        updated_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                    )
                ],
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
            ),
            TransactionResponse_BankAccountToMailedCheck(
                id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
                check_number=4005,
                status="COMPLETED",
                invoices=[
                    InvoiceResponse(
                        id="in_26e7b5d3-a739-4b23-9ad9-6aaa085f47a9",
                        status="PAID",
                        amount=100.0,
                        currency="USD",
                        invoice_date=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        deduction_date=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        settlement_date=datetime.datetime.fromisoformat(
                            "2021-01-03 00:00:00+00:00",
                        ),
                        due_date=datetime.datetime.fromisoformat(
                            "2021-01-31 00:00:00+00:00",
                        ),
                        invoice_number="INV-123",
                        note_to_self="For the month of January",
                        service_start_date=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        service_end_date=datetime.datetime.fromisoformat(
                            "2021-01-31 00:00:00+00:00",
                        ),
                        payer_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                        payer=CounterpartyResponse(
                            id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                            foreign_id="MY-DB-ID-12345",
                            name="Acme Inc.",
                            email="customer@acme.com",
                            accepted_tos=True,
                            status="verified",
                            is_customer=True,
                            is_payor=True,
                            is_payee=False,
                            is_network_payor=False,
                            is_network_payee=False,
                            account_type="business",
                            updated_at=datetime.datetime.fromisoformat(
                                "2024-01-02 00:00:00+00:00",
                            ),
                            created_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                            profile=ProfileResponse(
                                business=BusinessProfileResponse(
                                    email="customer@acme.com",
                                    legal_business_name="Acme Inc.",
                                    business_type="llc",
                                    phone=PhoneNumber(
                                        country_code="1",
                                        number="4155551234",
                                    ),
                                    address=Address(
                                        address_line_1="123 Main St",
                                        address_line_2="Unit 1",
                                        city="San Francisco",
                                        state_or_province="CA",
                                        postal_code="94105",
                                        country="US",
                                    ),
                                    tax_id_provided=True,
                                    tax_id=TaxId(
                                        ein=Ein(
                                            number="12-3456789",
                                        ),
                                    ),
                                    owners_provided=True,
                                ),
                            ),
                            accounts=[
                                CounterpartyCustomizationAccount(
                                    account_id="85866843",
                                    postal_code="94105",
                                    name_on_account="John Doe",
                                )
                            ],
                            payment_methods=[
                                PaymentMethodResponse_BankAccount(
                                    id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                                    account_name="My Checking Account",
                                    bank_name="Chase",
                                    routing_number="12345678",
                                    account_number="99988767623",
                                    account_type="CHECKING",
                                    status="VERIFIED",
                                    is_default_source=True,
                                    is_default_destination=True,
                                    supported_currencies=["USD"],
                                    metadata={},
                                    frozen=False,
                                    created_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                    updated_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                )
                            ],
                            counterparty_type=["ENTITY"],
                        ),
                        payment_source=PaymentMethodResponse_BankAccount(
                            id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                            account_name="My Checking Account",
                            bank_name="Chase",
                            routing_number="12345678",
                            account_number="99988767623",
                            account_type="CHECKING",
                            status="VERIFIED",
                            is_default_source=True,
                            is_default_destination=True,
                            supported_currencies=["USD"],
                            metadata={},
                            frozen=False,
                            created_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                            updated_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                        ),
                        payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                        vendor_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                        vendor=CounterpartyResponse(
                            id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                            foreign_id="MY-DB-ID-90909",
                            name="Big Box Store",
                            email="vendor@bigboxstore.com",
                            accepted_tos=False,
                            status="unverified",
                            is_customer=False,
                            is_payor=False,
                            is_payee=True,
                            is_network_payor=False,
                            is_network_payee=False,
                            account_type="business",
                            updated_at=datetime.datetime.fromisoformat(
                                "2024-01-02 00:00:00+00:00",
                            ),
                            created_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                            profile=ProfileResponse(
                                business=BusinessProfileResponse(
                                    email="vendor@bigboxstore.com",
                                    legal_business_name="Big Box Store",
                                    business_type="publicCorporation",
                                    tax_id_provided=False,
                                    owners_provided=False,
                                ),
                            ),
                            payment_methods=[
                                PaymentMethodResponse_BankAccount(
                                    id="pm_7610541f-4619-4033-8620-cfccfb811293",
                                    account_name="Vendor Checking Account",
                                    bank_name="Chase",
                                    routing_number="66554433",
                                    account_number="55934059697648",
                                    account_type="CHECKING",
                                    status="NEW",
                                    is_default_source=True,
                                    is_default_destination=True,
                                    supported_currencies=["USD"],
                                    metadata={},
                                    frozen=False,
                                    created_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                    updated_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                )
                            ],
                            counterparty_type=["ENTITY"],
                        ),
                        payment_destination=PaymentMethodResponse_Check(
                            id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
                            pay_to_the_order_of="John Doe",
                            address_line_1="123 Main St",
                            address_line_2="Apt 1",
                            city="New York",
                            state_or_province="NY",
                            postal_code="10001",
                            country="US",
                            is_default_source=False,
                            is_default_destination=True,
                            supported_currencies=["USD"],
                            metadata={},
                            frozen=False,
                            created_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                            updated_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                        ),
                        payment_destination_id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
                        payment_destination_options=PaymentDestinationOptions_Check(
                            delivery="MAIL",
                            print_description=True,
                        ),
                        payment_destination_confirmed=True,
                        transactions=[
                            TransactionResponseWithoutInvoices_BankAccountToBankAccount(
                                id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
                                status="COMPLETED",
                                created_at=datetime.datetime.fromisoformat(
                                    "2024-01-01 00:00:00+00:00",
                                ),
                                updated_at=datetime.datetime.fromisoformat(
                                    "2024-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        has_documents=True,
                        has_source_email=True,
                        comments=[
                            CommentResponse(
                                id="ic_b3525b66-da94-4525-9f31-426bcf657128",
                                text="This is an approval comment",
                                user=EntityUserResponse(
                                    id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
                                    foreign_id="MY-DB-ID-12345",
                                    email="john.doe@acme.com",
                                    name="John Doe",
                                    roles=["admin", "approver"],
                                    created_at=datetime.datetime.fromisoformat(
                                        "2024-01-01 00:00:00+00:00",
                                    ),
                                    updated_at=datetime.datetime.fromisoformat(
                                        "2024-01-01 00:00:00+00:00",
                                    ),
                                ),
                                associated_approval_action=AssociatedApprovalAction(
                                    user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                                    action="APPROVE",
                                ),
                                created_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                                updated_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        line_items=[
                            InvoiceLineItemResponse(
                                id="inli_26672f38-eb9a-48f1-a7a0-f1b855e38cd7",
                                amount=100.0,
                                currency="USD",
                                description="Product A",
                                name="Product A",
                                quantity=1.0,
                                unit_price=100.0,
                                category="EXPENSE",
                                service_start_date=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                                service_end_date=datetime.datetime.fromisoformat(
                                    "2021-01-31 00:00:00+00:00",
                                ),
                                metadata={"key1": "value1", "key2": "value2"},
                                gl_account_id="600394",
                                created_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                                updated_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        approvers=[
                            ApprovalSlot(
                                approval_policy_id="apvl_5ce50275-1789-42ea-bc60-bb7e6d03635c",
                                approval_slot_id="inap_9bb311c9-7c15-4c9e-8148-63814e0abec6",
                                assigned_user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                                action="APPROVE",
                                eligible_user_ids=[
                                    "user_e24fc81c-c5ee-47e8-af42-4fe29d895506"
                                ],
                                eligible_roles=["admin"],
                                date=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        approval_policy=[
                            ApprovalPolicyResponse(
                                id="apvl_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                                trigger=[
                                    Trigger_Amount(
                                        amount=100.0,
                                        currency="USD",
                                    )
                                ],
                                rule=Rule_Approver(
                                    num_approvers=2,
                                    identifier_list=IdentifierList_RolesList(
                                        value=["Admin", "Controller"]
                                    ),
                                ),
                                upstream_policy_id="root",
                                updated_at=datetime.datetime.fromisoformat(
                                    "2024-01-02 00:00:00+00:00",
                                ),
                                created_at=datetime.datetime.fromisoformat(
                                    "2024-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        metadata={"key1": "value1", "key2": "value2"},
                        foreign_id="YOUR-DATABASE-ID",
                        creator_user=EntityUserResponse(
                            id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
                            foreign_id="MY-DB-ID-12345",
                            email="john.doe@acme.com",
                            name="John Doe",
                            roles=["admin", "approver"],
                            created_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                            updated_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                        ),
                        processed_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        created_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        updated_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                    )
                ],
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
            ),
            TransactionResponse_BankAccountToBankAccount(
                id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
                status="FAILED",
                failure_reason=TransactionFailureReason(
                    code="R01",
                    description="The source bank account does not have sufficient funds",
                ),
                invoices=[
                    InvoiceResponse(
                        id="in_26e7b5d3-a739-4b23-9ad9-6aaa085f47a9",
                        status="FAILED",
                        payer_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                        payer=CounterpartyResponse(
                            id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                            foreign_id="MY-DB-ID-12345",
                            name="Acme Inc.",
                            email="customer@acme.com",
                            accepted_tos=True,
                            status="verified",
                            is_customer=True,
                            is_payor=True,
                            is_payee=False,
                            is_network_payor=False,
                            is_network_payee=False,
                            account_type="business",
                            updated_at=datetime.datetime.fromisoformat(
                                "2024-01-02 00:00:00+00:00",
                            ),
                            created_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                            profile=ProfileResponse(
                                business=BusinessProfileResponse(
                                    email="customer@acme.com",
                                    legal_business_name="Acme Inc.",
                                    business_type="llc",
                                    phone=PhoneNumber(
                                        country_code="1",
                                        number="4155551234",
                                    ),
                                    address=Address(
                                        address_line_1="123 Main St",
                                        address_line_2="Unit 1",
                                        city="San Francisco",
                                        state_or_province="CA",
                                        postal_code="94105",
                                        country="US",
                                    ),
                                    tax_id_provided=True,
                                    tax_id=TaxId(
                                        ein=Ein(
                                            number="12-3456789",
                                        ),
                                    ),
                                    owners_provided=True,
                                ),
                            ),
                            accounts=[
                                CounterpartyCustomizationAccount(
                                    account_id="85866843",
                                    postal_code="94105",
                                    name_on_account="John Doe",
                                )
                            ],
                            payment_methods=[
                                PaymentMethodResponse_BankAccount(
                                    id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                                    account_name="My Checking Account",
                                    bank_name="Chase",
                                    routing_number="12345678",
                                    account_number="99988767623",
                                    account_type="CHECKING",
                                    status="VERIFIED",
                                    is_default_source=True,
                                    is_default_destination=True,
                                    supported_currencies=["USD"],
                                    metadata={},
                                    frozen=False,
                                    created_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                    updated_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                )
                            ],
                            counterparty_type=["ENTITY"],
                        ),
                        vendor_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                        vendor=CounterpartyResponse(
                            id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                            foreign_id="MY-DB-ID-90909",
                            name="Big Box Store",
                            email="vendor@bigboxstore.com",
                            accepted_tos=False,
                            status="unverified",
                            is_customer=False,
                            is_payor=False,
                            is_payee=True,
                            is_network_payor=False,
                            is_network_payee=False,
                            account_type="business",
                            updated_at=datetime.datetime.fromisoformat(
                                "2024-01-02 00:00:00+00:00",
                            ),
                            created_at=datetime.datetime.fromisoformat(
                                "2024-01-01 00:00:00+00:00",
                            ),
                            profile=ProfileResponse(
                                business=BusinessProfileResponse(
                                    email="vendor@bigboxstore.com",
                                    legal_business_name="Big Box Store",
                                    business_type="publicCorporation",
                                    tax_id_provided=False,
                                    owners_provided=False,
                                ),
                            ),
                            payment_methods=[
                                PaymentMethodResponse_BankAccount(
                                    id="pm_7610541f-4619-4033-8620-cfccfb811293",
                                    account_name="Vendor Checking Account",
                                    bank_name="Chase",
                                    routing_number="66554433",
                                    account_number="55934059697648",
                                    account_type="CHECKING",
                                    status="NEW",
                                    is_default_source=True,
                                    is_default_destination=True,
                                    supported_currencies=["USD"],
                                    metadata={},
                                    frozen=False,
                                    created_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                    updated_at=datetime.datetime.fromisoformat(
                                        "2021-01-01 00:00:00+00:00",
                                    ),
                                )
                            ],
                            counterparty_type=["ENTITY"],
                        ),
                        currency="USD",
                        amount=100.0,
                        invoice_number="INV-123",
                        invoice_date=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        due_date=datetime.datetime.fromisoformat(
                            "2021-01-31 00:00:00+00:00",
                        ),
                        payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                        payment_source=PaymentMethodResponse_BankAccount(
                            id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                            account_name="My Checking Account",
                            bank_name="Chase",
                            routing_number="12345678",
                            account_number="99988767623",
                            account_type="CHECKING",
                            status="VERIFIED",
                            is_default_source=True,
                            is_default_destination=True,
                            supported_currencies=["USD"],
                            metadata={},
                            frozen=False,
                            created_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                            updated_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                        ),
                        payment_destination_id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
                        payment_destination=PaymentMethodResponse_Check(
                            id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
                            pay_to_the_order_of="John Doe",
                            address_line_1="123 Main St",
                            address_line_2="Apt 1",
                            city="New York",
                            state_or_province="NY",
                            postal_code="10001",
                            country="US",
                            is_default_source=False,
                            is_default_destination=True,
                            supported_currencies=["USD"],
                            metadata={},
                            frozen=False,
                            created_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                            updated_at=datetime.datetime.fromisoformat(
                                "2021-01-01 00:00:00+00:00",
                            ),
                        ),
                        deduction_date=datetime.datetime.fromisoformat(
                            "2021-01-29 00:00:00+00:00",
                        ),
                        payment_destination_confirmed=True,
                        has_documents=True,
                        has_source_email=True,
                        approvers=[],
                        approval_policy=[],
                        metadata={},
                        transactions=[
                            TransactionResponseWithoutInvoices_BankAccountToBankAccount(
                                id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
                                status="FAILED",
                                failure_reason=TransactionFailureReason(
                                    code="R01",
                                    description="The source bank account does not have sufficient funds",
                                ),
                                created_at=datetime.datetime.fromisoformat(
                                    "2024-01-01 00:00:00+00:00",
                                ),
                                updated_at=datetime.datetime.fromisoformat(
                                    "2024-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        created_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        updated_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                    )
                ],
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
            ),
        ],
    )
    """

    count: int
    has_more: bool = pydantic.Field(alias="hasMore")
    data: typing.List[TransactionResponse]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from ...payment_method_types.types.currency_code import CurrencyCode
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing
import pydantic


class AccelerationFundsBalanceResponse(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import AccelerationFundsBalanceResponse

    AccelerationFundsBalanceResponse(
        amount=100.0,
        currency="USD",
    )
    """

    amount: float
    currency: CurrencyCode

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .find_transactions_response import FindTransactionsResponse
from .transaction_failure_reason import TransactionFailureReason
from .transaction_id import TransactionId
from .transaction_response import (
    TransactionResponse,
    TransactionResponse_BankAccountToBankAccount,
    TransactionResponse_BankAccountToMailedCheck,
    TransactionResponse_Custom,
    TransactionResponse_OffPlatform,
)
from .transaction_response_bank_to_bank_base import TransactionResponseBankToBankBase
from .transaction_response_bank_to_bank_with_invoices import TransactionResponseBankToBankWithInvoices
from .transaction_response_bank_to_mailed_check_base import TransactionResponseBankToMailedCheckBase
from .transaction_response_bank_to_mailed_check_with_invoices import TransactionResponseBankToMailedCheckWithInvoices
from .transaction_response_base import TransactionResponseBase
from .transaction_response_custom_with_invoices import TransactionResponseCustomWithInvoices
from .transaction_response_without_invoices import (
    TransactionResponseWithoutInvoices,
    TransactionResponseWithoutInvoices_BankAccountToBankAccount,
    TransactionResponseWithoutInvoices_BankAccountToMailedCheck,
    TransactionResponseWithoutInvoices_Custom,
    TransactionResponseWithoutInvoices_OffPlatform,
)
from .transaction_status import TransactionStatus
from .transaction_type import TransactionType

__all__ = [
    "FindTransactionsResponse",
    "TransactionFailureReason",
    "TransactionId",
    "TransactionResponse",
    "TransactionResponseBankToBankBase",
    "TransactionResponseBankToBankWithInvoices",
    "TransactionResponseBankToMailedCheckBase",
    "TransactionResponseBankToMailedCheckWithInvoices",
    "TransactionResponseBase",
    "TransactionResponseCustomWithInvoices",
    "TransactionResponseWithoutInvoices",
    "TransactionResponseWithoutInvoices_BankAccountToBankAccount",
    "TransactionResponseWithoutInvoices_BankAccountToMailedCheck",
    "TransactionResponseWithoutInvoices_Custom",
    "TransactionResponseWithoutInvoices_OffPlatform",
    "TransactionResponse_BankAccountToBankAccount",
    "TransactionResponse_BankAccountToMailedCheck",
    "TransactionResponse_Custom",
    "TransactionResponse_OffPlatform",
    "TransactionStatus",
    "TransactionType",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class MetadataValidationRule_Regex(UniversalBaseModel):
    type: typing.Literal["regex"] = "regex"
    regex: str
    error_message: str = pydantic.Field(alias="errorMessage")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


MetadataValidationRule = MetadataValidationRule_Regex



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from .identifier_list import IdentifierList
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ApproverRule(UniversalBaseModel):
    num_approvers: int = pydantic.Field(alias="numApprovers")
    """
    Number of approvals required to approve an invoice
    """

    identifier_list: IdentifierList = pydantic.Field(alias="identifierList")
    """
    List of users or roles that should be used to determine eligible approvers
    """

    auto_assign: typing.Optional[bool] = pydantic.Field(alias="autoAssign", default=None)
    """
    If true, the policy will automatically assign approvers to the invoice. If more than one approver is eligible, the policy will assign all eligible approvers to the invoice.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...entity_types.types.entity_id import EntityId
from ...core.request_options import RequestOptions
from ...entity_types.types.email_template_response import EmailTemplateResponse
from ...core.jsonable_encoder import jsonable_encoder
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.pydantic_utilities import parse_obj_as
from ...commons.errors.bad_request import BadRequest
from ...commons.errors.unauthorized import Unauthorized
from ...commons.errors.forbidden import Forbidden
from ...commons.errors.not_found import NotFound
from ...commons.errors.conflict import Conflict
from ...commons.errors.internal_server_error import InternalServerError
from ...commons.errors.unimplemented import Unimplemented
from ...entity_types.types.email_template_request import EmailTemplateRequest
from ...entity_types.types.email_template_id import EmailTemplateId
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EmailTemplateClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all(
        self, entity_id: EntityId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[EmailTemplateResponse]:
        """
        Get all email templates for the entity

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[EmailTemplateResponse]

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.email_template.get_all(
            entity_id="ent_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-templates",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                typing.List[EmailTemplateResponse],
                parse_obj_as(
                    type_=typing.List[EmailTemplateResponse],  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        entity_id: EntityId,
        *,
        request: EmailTemplateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmailTemplateResponse:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        request : EmailTemplateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailTemplateResponse

        Examples
        --------
        from mercoa import Mercoa
        from mercoa.entity_types import EmailTemplateRequest

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.email_template.create(
            entity_id="ent_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
            request=EmailTemplateRequest(
                template_type="PAYMENT",
                name="Generic Payment Email",
                subject="Action Required - Your payment is due",
                content="<h1>Your invoice has been sent.</h1>",
                is_default=True,
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-template",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EmailTemplateResponse,
                parse_obj_as(
                    type_=EmailTemplateResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        entity_id: EntityId,
        email_template_id: EmailTemplateId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmailTemplateResponse:
        """
        Get entity email template

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        email_template_id : EmailTemplateId
            Email Template ID or Email Template ForeignID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailTemplateResponse

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.email_template.get(
            entity_id="ent_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
            email_template_id="emt_8545a84e-a45f-41bf-bdf1-33b42a55812c",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-template/{jsonable_encoder(email_template_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EmailTemplateResponse,
                parse_obj_as(
                    type_=EmailTemplateResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        entity_id: EntityId,
        email_template_id: EmailTemplateId,
        *,
        request: EmailTemplateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmailTemplateResponse:
        """
        Update entity email template

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        email_template_id : EmailTemplateId
            Email Template ID or Email Template ForeignID

        request : EmailTemplateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailTemplateResponse

        Examples
        --------
        from mercoa import Mercoa
        from mercoa.entity_types import EmailTemplateRequest

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.email_template.update(
            entity_id="ent_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
            email_template_id="emt_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            request=EmailTemplateRequest(
                template_type="PAYMENT",
                name="Generic Payment Email",
                subject="Action Required - Your payment is due",
                content="<h1>Your invoice has been sent.</h1>",
                is_default=True,
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-template/{jsonable_encoder(email_template_id)}",
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EmailTemplateResponse,
                parse_obj_as(
                    type_=EmailTemplateResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        entity_id: EntityId,
        email_template_id: EmailTemplateId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete entity email template. This will also remove the email template from all entities.

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        email_template_id : EmailTemplateId
            Email Template ID or Email Template ForeignID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.email_template.delete(
            entity_id="string",
            email_template_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-template/{jsonable_encoder(email_template_id)}",
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncEmailTemplateClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all(
        self, entity_id: EntityId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[EmailTemplateResponse]:
        """
        Get all email templates for the entity

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[EmailTemplateResponse]

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.email_template.get_all(
                entity_id="ent_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-templates",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                typing.List[EmailTemplateResponse],
                parse_obj_as(
                    type_=typing.List[EmailTemplateResponse],  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        entity_id: EntityId,
        *,
        request: EmailTemplateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmailTemplateResponse:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        request : EmailTemplateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailTemplateResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa
        from mercoa.entity_types import EmailTemplateRequest

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.email_template.create(
                entity_id="ent_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
                request=EmailTemplateRequest(
                    template_type="PAYMENT",
                    name="Generic Payment Email",
                    subject="Action Required - Your payment is due",
                    content="<h1>Your invoice has been sent.</h1>",
                    is_default=True,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-template",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EmailTemplateResponse,
                parse_obj_as(
                    type_=EmailTemplateResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        entity_id: EntityId,
        email_template_id: EmailTemplateId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmailTemplateResponse:
        """
        Get entity email template

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        email_template_id : EmailTemplateId
            Email Template ID or Email Template ForeignID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailTemplateResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.email_template.get(
                entity_id="ent_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
                email_template_id="emt_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-template/{jsonable_encoder(email_template_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EmailTemplateResponse,
                parse_obj_as(
                    type_=EmailTemplateResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        entity_id: EntityId,
        email_template_id: EmailTemplateId,
        *,
        request: EmailTemplateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmailTemplateResponse:
        """
        Update entity email template

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        email_template_id : EmailTemplateId
            Email Template ID or Email Template ForeignID

        request : EmailTemplateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailTemplateResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa
        from mercoa.entity_types import EmailTemplateRequest

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.email_template.update(
                entity_id="ent_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
                email_template_id="emt_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                request=EmailTemplateRequest(
                    template_type="PAYMENT",
                    name="Generic Payment Email",
                    subject="Action Required - Your payment is due",
                    content="<h1>Your invoice has been sent.</h1>",
                    is_default=True,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-template/{jsonable_encoder(email_template_id)}",
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EmailTemplateResponse,
                parse_obj_as(
                    type_=EmailTemplateResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        entity_id: EntityId,
        email_template_id: EmailTemplateId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete entity email template. This will also remove the email template from all entities.

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        email_template_id : EmailTemplateId
            Email Template ID or Email Template ForeignID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.email_template.delete(
                entity_id="string",
                email_template_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/email-template/{jsonable_encoder(email_template_id)}",
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .transaction_failure_reason import TransactionFailureReason
import pydantic
from .transaction_id import TransactionId
from .transaction_status import TransactionStatus
import datetime as dt
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TransactionResponseWithoutInvoices_BankAccountToBankAccount(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.transaction import (
        TransactionResponseWithoutInvoices_BankAccountToBankAccount,
    )

    TransactionResponseWithoutInvoices_BankAccountToBankAccount(
        id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
        status="PENDING",
        created_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
    )
    """

    type: typing.Literal["bankAccountToBankAccount"] = "bankAccountToBankAccount"
    failure_reason: typing.Optional[TransactionFailureReason] = pydantic.Field(alias="failureReason", default=None)
    id: TransactionId
    status: TransactionStatus
    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class TransactionResponseWithoutInvoices_BankAccountToMailedCheck(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.transaction import (
        TransactionResponseWithoutInvoices_BankAccountToBankAccount,
    )

    TransactionResponseWithoutInvoices_BankAccountToBankAccount(
        id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
        status="PENDING",
        created_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
    )
    """

    type: typing.Literal["bankAccountToMailedCheck"] = "bankAccountToMailedCheck"
    check_number: int = pydantic.Field(alias="checkNumber")
    id: TransactionId
    status: TransactionStatus
    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class TransactionResponseWithoutInvoices_Custom(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.transaction import (
        TransactionResponseWithoutInvoices_BankAccountToBankAccount,
    )

    TransactionResponseWithoutInvoices_BankAccountToBankAccount(
        id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
        status="PENDING",
        created_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
    )
    """

    type: typing.Literal["custom"] = "custom"
    id: TransactionId
    status: TransactionStatus
    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class TransactionResponseWithoutInvoices_OffPlatform(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.transaction import (
        TransactionResponseWithoutInvoices_BankAccountToBankAccount,
    )

    TransactionResponseWithoutInvoices_BankAccountToBankAccount(
        id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
        status="PENDING",
        created_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
    )
    """

    type: typing.Literal["offPlatform"] = "offPlatform"
    id: TransactionId
    status: TransactionStatus
    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
import datetime

from mercoa.transaction import (
    TransactionResponseWithoutInvoices_BankAccountToBankAccount,
)

TransactionResponseWithoutInvoices_BankAccountToBankAccount(
    id="trx_bb08e72f-19f8-45f3-bcf9-46fdc46cb2f4",
    status="PENDING",
    created_at=datetime.datetime.fromisoformat(
        "2024-01-01 00:00:00+00:00",
    ),
    updated_at=datetime.datetime.fromisoformat(
        "2024-01-01 00:00:00+00:00",
    ),
)
"""
TransactionResponseWithoutInvoices = typing.Union[
    TransactionResponseWithoutInvoices_BankAccountToBankAccount,
    TransactionResponseWithoutInvoices_BankAccountToMailedCheck,
    TransactionResponseWithoutInvoices_Custom,
    TransactionResponseWithoutInvoices_OffPlatform,
]



# This file was auto-generated by Fern from our API Definition.

from .transaction_response_base import TransactionResponseBase
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class TransactionResponseBankToMailedCheckBase(TransactionResponseBase):
    check_number: int = pydantic.Field(alias="checkNumber")
    """
    The number of the check
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class UtilityPaymentDestinationOptions(UniversalBaseModel):
    account_id: str = pydantic.Field(alias="accountId")
    """
    The ID for the utility account to pay with. Links to accounts listed on payor/payee relationship.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .invoice_status import InvoiceStatus
import pydantic
from ...payment_method_types.types.currency_code import CurrencyCode
import datetime as dt
from ...entity_types.types.entity_id import EntityId
from ...payment_method_types.types.payment_method_id import PaymentMethodId
from .payment_destination_options import PaymentDestinationOptions
from .approval_slot_assignment import ApprovalSlotAssignment
from ...entity_types.types.entity_user_id import EntityUserId
from .invoice_fees_request import InvoiceFeesRequest
from .payment_schedule import PaymentSchedule
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceTemplateRequestBase(UniversalBaseModel):
    status: typing.Optional[InvoiceStatus] = None
    amount: typing.Optional[float] = pydantic.Field(default=None)
    """
    Total amount of invoice in major units. If the entered amount has more decimal places than the currency supports, trailing decimals will be truncated.
    """

    currency: typing.Optional[CurrencyCode] = pydantic.Field(default=None)
    """
    Currency code for the amount. Defaults to USD.
    """

    invoice_date: typing.Optional[dt.datetime] = pydantic.Field(alias="invoiceDate", default=None)
    """
    Date the invoice was issued.
    """

    deduction_date: typing.Optional[dt.datetime] = pydantic.Field(alias="deductionDate", default=None)
    """
    Initial date when funds are scheduled to be deducted from payer's account.
    """

    due_date: typing.Optional[dt.datetime] = pydantic.Field(alias="dueDate", default=None)
    """
    Due date of invoice.
    """

    invoice_number: typing.Optional[str] = pydantic.Field(alias="invoiceNumber", default=None)
    note_to_self: typing.Optional[str] = pydantic.Field(alias="noteToSelf", default=None)
    """
    Note to self or memo on invoice.
    """

    service_start_date: typing.Optional[dt.datetime] = pydantic.Field(alias="serviceStartDate", default=None)
    service_end_date: typing.Optional[dt.datetime] = pydantic.Field(alias="serviceEndDate", default=None)
    payer_id: typing.Optional[EntityId] = pydantic.Field(alias="payerId", default=None)
    """
    ID or foreign ID of the payer of this invoice.
    """

    payment_source_id: typing.Optional[PaymentMethodId] = pydantic.Field(alias="paymentSourceId", default=None)
    """
    ID of payment source for this invoice. If not provided, will attempt to use the default payment source for the payer when creating an invoice if a default payment source exists for the payer.
    """

    vendor_id: typing.Optional[EntityId] = pydantic.Field(alias="vendorId", default=None)
    """
    ID or foreign ID of the vendor of this invoice.
    """

    payment_destination_id: typing.Optional[PaymentMethodId] = pydantic.Field(
        alias="paymentDestinationId", default=None
    )
    """
    ID of payment destination for this invoice. If not provided, will attempt to use the default payment destination for the vendor when creating an invoice if a default payment destination exists for the vendor.
    """

    payment_destination_options: typing.Optional[PaymentDestinationOptions] = pydantic.Field(
        alias="paymentDestinationOptions", default=None
    )
    """
    Options for the payment destination. Depending on the payment destination, this may include things such as check delivery method.
    """

    approvers: typing.Optional[typing.List[ApprovalSlotAssignment]] = pydantic.Field(default=None)
    """
    Set approvers for this invoice.
    """

    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Metadata associated with this invoice.
    """

    document: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base64 encoded image or PDF of invoice document. PNG, JPG, WEBP, and PDF are supported. 10MB max. If the invoice already has a document, this will add a new document to the invoice.
    """

    creator_user_id: typing.Optional[EntityUserId] = pydantic.Field(alias="creatorUserId", default=None)
    """
    User ID or Foreign ID of entity user who created this invoice.
    """

    fees: typing.Optional[InvoiceFeesRequest] = pydantic.Field(default=None)
    """
    If using a custom payment method, you can override the default fees for this invoice. If not provided, the default fees for the custom payment method will be used.
    """

    batch_payment: typing.Optional[bool] = pydantic.Field(alias="batchPayment", default=None)
    """
    If true, this invoice will be paid as a batch payment. Batches are automatically determined by Mercoa based on the payment source, destination, and scheduled payment date.
    """

    payment_schedule: typing.Optional[PaymentSchedule] = pydantic.Field(alias="paymentSchedule", default=None)
    """
    If this is a recurring invoice, this will be the payment schedule for the invoice. If not provided, this will be a one-time invoice.
    """

    tax_amount: typing.Optional[float] = pydantic.Field(alias="taxAmount", default=None)
    """
    Tax amount for this invoice.
    """

    shipping_amount: typing.Optional[float] = pydantic.Field(alias="shippingAmount", default=None)
    """
    Shipping amount for this invoice.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .vendor_credit_id import VendorCreditId
import typing
import pydantic
from ...payment_method_types.types.currency_code import CurrencyCode
from ...entity_types.types.entity_id import EntityId
from ...invoice_types.types.invoice_id import InvoiceId
import datetime as dt
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class VendorCreditResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.vendor_credit_types import VendorCreditResponse

    VendorCreditResponse(
        id="vcr_c3f4c87d-794d-4543-9562-575cdddfc0d7",
        total_amount=100.0,
        remaining_amount=100.0,
        currency="USD",
        vendor_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
        payer_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
        creator_entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
        note="This is a note",
        invoice_ids=["in_26e7b5d3-a739-4b23-9ad9-6aaa085f47a9"],
        created_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
    )
    """

    id: VendorCreditId
    total_amount: typing.Optional[float] = pydantic.Field(alias="totalAmount", default=None)
    """
    Total issued amount of the vendor credit in major units
    """

    remaining_amount: typing.Optional[float] = pydantic.Field(alias="remainingAmount", default=None)
    """
    Remaining usable amount in the vendor credit in major units
    """

    currency: typing.Optional[CurrencyCode] = pydantic.Field(default=None)
    """
    Currency code for the amount. Defaults to USD.
    """

    vendor_id: EntityId = pydantic.Field(alias="vendorId")
    """
    ID of the vendor the vendor credit may be used for
    """

    payer_id: EntityId = pydantic.Field(alias="payerId")
    """
    ID of the payer who may use the vendor credit
    """

    creator_entity_id: typing.Optional[EntityId] = pydantic.Field(alias="creatorEntityId", default=None)
    """
    ID of the entity that created this vendor credit
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional note to attach to the vendor credit
    """

    invoice_ids: typing.Optional[typing.List[InvoiceId]] = pydantic.Field(alias="invoiceIds", default=None)
    """
    The IDs of the invoices that this vendor credit has been applied to
    """

    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...payment_method_types.types.currency_code import CurrencyCode
import typing
from .vendor_credit_response import VendorCreditResponse
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CalculateVendorCreditUsageResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.vendor_credit_types import (
        CalculateVendorCreditUsageResponse,
        VendorCreditResponse,
    )

    CalculateVendorCreditUsageResponse(
        remaining_amount=50.0,
        currency="USD",
        vendor_credits=[
            VendorCreditResponse(
                id="vcr_c3f4c87d-794d-4543-9562-575cdddfc0d7",
                total_amount=100.0,
                remaining_amount=0.0,
                currency="USD",
                vendor_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                payer_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                creator_entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                note="This is a note",
                invoice_ids=["in_26e7b5d3-a739-4b23-9ad9-6aaa085f47a9"],
                created_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
            )
        ],
    )
    """

    remaining_amount: float = pydantic.Field(alias="remainingAmount")
    """
    Remaining amount on the invoice after vendor credits are applied in major units.
    """

    currency: CurrencyCode = pydantic.Field()
    """
    Currency code for the amount. Defaults to USD.
    """

    vendor_credits: typing.List[VendorCreditResponse] = pydantic.Field(alias="vendorCredits")
    """
    Vendor credits that will be applied to the invoice. The objects returned represent the states each vendor credit will be in AFTER they are applied to the invoice, not their current states.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .transaction_response_base import TransactionResponseBase
import typing
from .transaction_failure_reason import TransactionFailureReason
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TransactionResponseBankToBankBase(TransactionResponseBase):
    failure_reason: typing.Optional[TransactionFailureReason] = pydantic.Field(alias="failureReason", default=None)
    """
    If the invoice failed to be paid, this field will be populated with the reason of failure.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...payment_method_types.types.currency_code import CurrencyCode
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class VendorCreditRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.vendor_credit_types import VendorCreditRequest

    VendorCreditRequest(
        total_amount=100.0,
        currency="USD",
        note="This is a note",
    )
    """

    total_amount: float = pydantic.Field(alias="totalAmount")
    """
    Total amount of the vendor credit in major units
    """

    currency: CurrencyCode = pydantic.Field()
    """
    Currency code for the amount. Defaults to USD.
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional note to attach to the vendor credit
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .common_onboarding_options import CommonOnboardingOptions
from .onboarding_option import OnboardingOption
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class IndividualOnboardingOptions(CommonOnboardingOptions):
    date_of_birth: OnboardingOption = pydantic.Field(alias="dateOfBirth")
    ssn: OnboardingOption

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

TransactionStatus = typing.Union[
    typing.Literal["CREATED", "PENDING", "COMPLETED", "FAILED", "REVERSED", "QUEUED", "CANCELED"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

TransactionType = typing.Union[
    typing.Literal["bankAccountToBankAccount", "bankAccountToMailedCheck", "custom", "offPlatform"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class NotificationCustomizationRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import NotificationCustomizationRequest

    NotificationCustomizationRequest(
        assume_role="admin",
    )
    """

    assume_role: typing.Optional[str] = pydantic.Field(alias="assumeRole", default=None)
    """
    If set, notifications to this role will be sent to the email address of the entity. Set as empty string to disable.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
